@@ -144,13 +144,51 @@ AWS_STATIC_STRING_FROM_LITERAL(
     "\"Expiration\":\"2020-02-25T06:03:31Z\"}'");
 #endif
 
+#ifdef _WIN32
+AWS_STATIC_STRING_FROM_LITERAL(
+    s_test_command_with_account_id,
+    "echo {\"Version\": 1, \"AccessKeyId\": \"AccessKey123\", "
+    "\"SecretAccessKey\": \"SecretAccessKey321\", \"SessionToken\":\"TokenSuccess\", "
+    "\"Expiration\":\"2020-02-25T06:03:31Z\", \"AccountId\":\"AccountId123\"}");
+#else
+AWS_STATIC_STRING_FROM_LITERAL(
+    s_test_command_with_account_id,
+    "echo '{\"Version\": 1, \"AccessKeyId\": \"AccessKey123\", "
+    "\"SecretAccessKey\": \"SecretAccessKey321\", \"SessionToken\":\"TokenSuccess\", "
+    "\"Expiration\":\"2020-02-25T06:03:31Z\", \"AccountId\":\"AccountId123\"}'");
+#endif
+
+#ifdef _WIN32
+AWS_STATIC_STRING_FROM_LITERAL(
+    s_test_command_with_logging_on_stderr,
+    "("
+    "echo Logging on stderr >&2"
+    " && "
+    "echo {\"Version\": 1, \"AccessKeyId\": \"AccessKey123\", "
+    "\"SecretAccessKey\": \"SecretAccessKey321\", \"SessionToken\":\"TokenSuccess\", "
+    "\"Expiration\":\"2020-02-25T06:03:31Z\"}"
+    ")");
+#else
+AWS_STATIC_STRING_FROM_LITERAL(
+    s_test_command_with_logging_on_stderr,
+    "("
+    "echo 'Logging on stderr' >&2"
+    " && "
+    "echo '{\"Version\": 1, \"AccessKeyId\": \"AccessKey123\", "
+    "\"SecretAccessKey\": \"SecretAccessKey321\", \"SessionToken\":\"TokenSuccess\", "
+    "\"Expiration\":\"2020-02-25T06:03:31Z\"}'"
+    ")");
+#endif
+
 AWS_STATIC_STRING_FROM_LITERAL(s_bad_test_command, "/i/dont/know/what/is/this/command");
 AWS_STATIC_STRING_FROM_LITERAL(s_bad_command_output, "echo \"Hello, World!\"");
 
 AWS_STATIC_STRING_FROM_LITERAL(s_good_access_key_id, "AccessKey123");
 AWS_STATIC_STRING_FROM_LITERAL(s_good_secret_access_key, "SecretAccessKey321");
 AWS_STATIC_STRING_FROM_LITERAL(s_good_session_token, "TokenSuccess");
-AWS_STATIC_STRING_FROM_LITERAL(s_good_expiration, "2020-02-25T06:03:31Z");
+AWS_STATIC_STRING_FROM_LITERAL(s_good_account_id, "AccountId123");
+
+static uint64_t s_good_expiration = 1582610611;
 
 AWS_STATIC_STRING_FROM_LITERAL(
     s_process_config_file_contents,
@@ -262,15 +300,14 @@ static int s_credentials_provider_process_new_failed(struct aws_allocator *alloc
 }
 AWS_TEST_CASE(credentials_provider_process_new_failed, s_credentials_provider_process_new_failed);
 
-static int s_credentials_provider_process_bad_command(struct aws_allocator *allocator, void *ctx) {
-    (void)ctx;
+static int s_test_command_expect_failure(struct aws_allocator *allocator, const struct aws_string *command) {
 
     s_aws_process_tester_init(allocator);
 
     struct aws_byte_buf content_buf;
     struct aws_byte_buf existing_content = aws_byte_buf_from_c_str(aws_string_c_str(s_process_config_file_contents));
     aws_byte_buf_init_copy(&content_buf, allocator, &existing_content);
-    struct aws_byte_cursor cursor = aws_byte_cursor_from_string(s_bad_test_command);
+    struct aws_byte_cursor cursor = aws_byte_cursor_from_string(command);
     ASSERT_TRUE(aws_byte_buf_append_dynamic(&content_buf, &cursor) == AWS_OP_SUCCESS);
     cursor = aws_byte_cursor_from_c_str("\n");
     ASSERT_TRUE(aws_byte_buf_append_dynamic(&content_buf, &cursor) == AWS_OP_SUCCESS);
@@ -304,78 +341,66 @@ static int s_credentials_provider_process_bad_command(struct aws_allocator *allo
     s_aws_process_tester_cleanup();
     return 0;
 }
+
+static int s_credentials_provider_process_bad_command(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+    return s_test_command_expect_failure(allocator, s_bad_test_command);
+}
 AWS_TEST_CASE(credentials_provider_process_bad_command, s_credentials_provider_process_bad_command);
 
 static int s_credentials_provider_process_incorrect_command_output(struct aws_allocator *allocator, void *ctx) {
     (void)ctx;
-
-    s_aws_process_tester_init(allocator);
-
-    struct aws_byte_buf content_buf;
-    struct aws_byte_buf existing_content = aws_byte_buf_from_c_str(aws_string_c_str(s_process_config_file_contents));
-    aws_byte_buf_init_copy(&content_buf, allocator, &existing_content);
-    struct aws_byte_cursor cursor = aws_byte_cursor_from_string(s_bad_command_output);
-    ASSERT_TRUE(aws_byte_buf_append_dynamic(&content_buf, &cursor) == AWS_OP_SUCCESS);
-    cursor = aws_byte_cursor_from_c_str("\n");
-    ASSERT_TRUE(aws_byte_buf_append_dynamic(&content_buf, &cursor) == AWS_OP_SUCCESS);
-
-    struct aws_string *config_file_contents = aws_string_new_from_array(allocator, content_buf.buffer, content_buf.len);
-    ASSERT_TRUE(config_file_contents != NULL);
-    aws_byte_buf_clean_up(&content_buf);
-
-    s_aws_process_test_init_config_profile(allocator, config_file_contents);
-    aws_string_destroy(config_file_contents);
-
-    struct aws_credentials_provider_process_options options = {
-        .shutdown_options =
-            {
-                .shutdown_callback = s_on_shutdown_complete,
-                .shutdown_user_data = NULL,
-            },
-        .profile_to_use = aws_byte_cursor_from_string(s_credentials_process_profile),
-    };
-    struct aws_credentials_provider *provider = aws_credentials_provider_new_process(allocator, &options);
-    ASSERT_NOT_NULL(provider);
-    aws_credentials_provider_get_credentials(provider, s_get_credentials_callback, NULL);
-
-    s_aws_wait_for_credentials_result();
-
-    ASSERT_TRUE(s_tester.has_received_credentials_callback == true);
-    ASSERT_TRUE(s_tester.credentials == NULL);
-
-    aws_credentials_provider_release(provider);
-    s_aws_wait_for_provider_shutdown_callback();
-    s_aws_process_tester_cleanup();
-    return 0;
+    return s_test_command_expect_failure(allocator, s_bad_command_output);
 }
 AWS_TEST_CASE(
     credentials_provider_process_incorrect_command_output,
     s_credentials_provider_process_incorrect_command_output);
 
-static int s_verify_credentials(struct aws_credentials *credentials) {
+static int s_verify_credentials(struct aws_credentials *credentials, struct aws_credentials *expected_credentials) {
     ASSERT_NOT_NULL(credentials);
-    ASSERT_CURSOR_VALUE_STRING_EQUALS(aws_credentials_get_access_key_id(credentials), s_good_access_key_id);
-    ASSERT_CURSOR_VALUE_STRING_EQUALS(aws_credentials_get_secret_access_key(credentials), s_good_secret_access_key);
-    ASSERT_CURSOR_VALUE_STRING_EQUALS(aws_credentials_get_session_token(credentials), s_good_session_token);
+    ASSERT_NOT_NULL(expected_credentials);
 
-    struct aws_date_time expiration;
-    struct aws_byte_cursor date_cursor = aws_byte_cursor_from_string(s_good_expiration);
-    aws_date_time_init_from_str_cursor(&expiration, &date_cursor, AWS_DATE_FORMAT_ISO_8601);
-    ASSERT_TRUE(
-        aws_credentials_get_expiration_timepoint_seconds(s_tester.credentials) == (uint64_t)expiration.timestamp);
+    ASSERT_BIN_ARRAYS_EQUALS(
+        aws_credentials_get_access_key_id(credentials).ptr,
+        aws_credentials_get_access_key_id(credentials).len,
+        aws_credentials_get_access_key_id(expected_credentials).ptr,
+        aws_credentials_get_access_key_id(expected_credentials).len);
+
+    ASSERT_BIN_ARRAYS_EQUALS(
+        aws_credentials_get_secret_access_key(credentials).ptr,
+        aws_credentials_get_secret_access_key(credentials).len,
+        aws_credentials_get_secret_access_key(expected_credentials).ptr,
+        aws_credentials_get_secret_access_key(expected_credentials).len);
+
+    ASSERT_BIN_ARRAYS_EQUALS(
+        aws_credentials_get_session_token(credentials).ptr,
+        aws_credentials_get_session_token(credentials).len,
+        aws_credentials_get_session_token(expected_credentials).ptr,
+        aws_credentials_get_session_token(expected_credentials).len);
+
+    ASSERT_BIN_ARRAYS_EQUALS(
+        aws_credentials_get_account_id(credentials).ptr,
+        aws_credentials_get_account_id(credentials).len,
+        aws_credentials_get_account_id(expected_credentials).ptr,
+        aws_credentials_get_account_id(expected_credentials).len);
+
+    ASSERT_UINT_EQUALS(
+        aws_credentials_get_expiration_timepoint_seconds(credentials),
+        aws_credentials_get_expiration_timepoint_seconds(expected_credentials));
 
     return AWS_OP_SUCCESS;
 }
 
-static int s_credentials_provider_process_basic_success(struct aws_allocator *allocator, void *ctx) {
-    (void)ctx;
-
+static int s_test_command_expect_success(
+    struct aws_allocator *allocator,
+    const struct aws_string *command,
+    struct aws_credentials *expected_credentials) {
     s_aws_process_tester_init(allocator);
 
     struct aws_byte_buf content_buf;
     struct aws_byte_buf existing_content = aws_byte_buf_from_c_str(aws_string_c_str(s_process_config_file_contents));
     aws_byte_buf_init_copy(&content_buf, allocator, &existing_content);
-    struct aws_byte_cursor cursor = aws_byte_cursor_from_string(s_test_command);
+    struct aws_byte_cursor cursor = aws_byte_cursor_from_string(command);
     ASSERT_TRUE(aws_byte_buf_append_dynamic(&content_buf, &cursor) == AWS_OP_SUCCESS);
     cursor = aws_byte_cursor_from_c_str("\n");
     ASSERT_TRUE(aws_byte_buf_append_dynamic(&content_buf, &cursor) == AWS_OP_SUCCESS);
@@ -402,15 +427,71 @@ static int s_credentials_provider_process_basic_success(struct aws_allocator *al
     s_aws_wait_for_credentials_result();
 
     ASSERT_TRUE(s_tester.has_received_credentials_callback == true);
-    ASSERT_SUCCESS(s_verify_credentials(s_tester.credentials));
+    ASSERT_SUCCESS(s_verify_credentials(s_tester.credentials, expected_credentials));
 
     aws_credentials_provider_release(provider);
     s_aws_wait_for_provider_shutdown_callback();
     s_aws_process_tester_cleanup();
     return 0;
 }
+
+static int s_credentials_provider_process_basic_success(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+    struct aws_credentials *expected_credentials = aws_credentials_new_from_string(
+        allocator, s_good_access_key_id, s_good_secret_access_key, s_good_session_token, s_good_expiration);
+    ASSERT_SUCCESS(s_test_command_expect_success(allocator, s_test_command, expected_credentials));
+    aws_credentials_release(expected_credentials);
+    return AWS_OP_SUCCESS;
+}
 AWS_TEST_CASE(credentials_provider_process_basic_success, s_credentials_provider_process_basic_success);
 
+static int s_credentials_provider_process_basic_success_without_session_token(
+    struct aws_allocator *allocator,
+    void *ctx) {
+    (void)ctx;
+    struct aws_credentials *expected_credentials = aws_credentials_new_from_string(
+        allocator, s_good_access_key_id, s_good_secret_access_key, NULL, s_good_expiration);
+    ASSERT_SUCCESS(s_test_command_expect_success(allocator, s_test_command_without_token, expected_credentials));
+    aws_credentials_release(expected_credentials);
+    return AWS_OP_SUCCESS;
+}
+AWS_TEST_CASE(
+    credentials_provider_process_basic_success_without_session_token,
+    s_credentials_provider_process_basic_success_without_session_token);
+
+static int s_credentials_provider_process_basic_success_with_account_id(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+    struct aws_credentials_options creds_option = {
+        .access_key_id_cursor = aws_byte_cursor_from_string(s_good_access_key_id),
+        .secret_access_key_cursor = aws_byte_cursor_from_string(s_good_secret_access_key),
+        .session_token_cursor = aws_byte_cursor_from_string(s_good_session_token),
+        .account_id_cursor = aws_byte_cursor_from_string(s_good_account_id),
+        .expiration_timepoint_seconds = s_good_expiration,
+    };
+    struct aws_credentials *expected_credentials = aws_credentials_new_with_options(allocator, &creds_option);
+    ASSERT_SUCCESS(s_test_command_expect_success(allocator, s_test_command_with_account_id, expected_credentials));
+    aws_credentials_release(expected_credentials);
+    return AWS_OP_SUCCESS;
+}
+AWS_TEST_CASE(
+    credentials_provider_process_basic_success_with_account_id,
+    s_credentials_provider_process_basic_success_with_account_id);
+
+/* Test that stderr is ignored, if the process otherwise succeeds with exit code 0 and valid JSON to stdout.
+ * Once upon a time stderr and stdout were merged, and mundane logging to stderr would break things. */
+static int s_credentials_provider_process_success_ignores_stderr(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+    struct aws_credentials *expected_credentials = aws_credentials_new_from_string(
+        allocator, s_good_access_key_id, s_good_secret_access_key, s_good_session_token, s_good_expiration);
+    ASSERT_SUCCESS(
+        s_test_command_expect_success(allocator, s_test_command_with_logging_on_stderr, expected_credentials));
+    aws_credentials_release(expected_credentials);
+    return AWS_OP_SUCCESS;
+}
+AWS_TEST_CASE(
+    credentials_provider_process_success_ignores_stderr,
+    s_credentials_provider_process_success_ignores_stderr);
+
 static int s_credentials_provider_process_basic_success_from_profile_provider(
     struct aws_allocator *allocator,
     void *ctx) {
@@ -448,7 +529,10 @@ static int s_credentials_provider_process_basic_success_from_profile_provider(
     s_aws_wait_for_credentials_result();
 
     ASSERT_TRUE(s_tester.has_received_credentials_callback == true);
-    ASSERT_SUCCESS(s_verify_credentials(s_tester.credentials));
+    struct aws_credentials *expected_credentials = aws_credentials_new_from_string(
+        allocator, s_good_access_key_id, s_good_secret_access_key, s_good_session_token, s_good_expiration);
+    ASSERT_SUCCESS(s_verify_credentials(s_tester.credentials, expected_credentials));
+    aws_credentials_release(expected_credentials);
 
     aws_credentials_provider_release(provider);
     s_aws_wait_for_provider_shutdown_callback();
@@ -518,7 +602,10 @@ static int s_credentials_provider_process_basic_success_cached(struct aws_alloca
     s_aws_wait_for_credentials_result();
 
     ASSERT_TRUE(s_tester.has_received_credentials_callback == true);
-    ASSERT_SUCCESS(s_verify_credentials(s_tester.credentials));
+    struct aws_credentials *expected_credentials = aws_credentials_new_from_string(
+        allocator, s_good_access_key_id, s_good_secret_access_key, s_good_session_token, s_good_expiration);
+    ASSERT_SUCCESS(s_verify_credentials(s_tester.credentials, expected_credentials));
+    aws_credentials_release(expected_credentials);
 
     aws_string_destroy(config_file_path);
     aws_profile_collection_release(profile_collection);
@@ -528,3 +615,74 @@ static int s_credentials_provider_process_basic_success_cached(struct aws_alloca
     return 0;
 }
 AWS_TEST_CASE(credentials_provider_process_basic_success_cached, s_credentials_provider_process_basic_success_cached);
+
+AWS_STATIC_STRING_FROM_LITERAL(s_profile_access_key_id, "ProfileAccessKey");
+AWS_STATIC_STRING_FROM_LITERAL(s_profile_secret_access_key, "ProfileSecretKey");
+
+AWS_STATIC_STRING_FROM_LITERAL(
+    s_config_with_both_profile_and_process_creds,
+    "[profile default]\n"
+    "region=us-east-1\n"
+    "aws_access_key_id=ProfileAccessKey\n"
+    "aws_secret_access_key=ProfileSecretKey\n"
+    "credential_process=");
+
+static int s_credentials_provider_profile_takes_priority_over_process(struct aws_allocator *allocator, void *ctx) {
+    (void)ctx;
+
+    s_aws_process_tester_init(allocator);
+
+    struct aws_byte_buf content_buf;
+    struct aws_byte_buf existing_content =
+        aws_byte_buf_from_c_str(aws_string_c_str(s_config_with_both_profile_and_process_creds));
+    aws_byte_buf_init_copy(&content_buf, allocator, &existing_content);
+    struct aws_byte_cursor cursor = aws_byte_cursor_from_string(s_test_command);
+    ASSERT_TRUE(aws_byte_buf_append_dynamic(&content_buf, &cursor) == AWS_OP_SUCCESS);
+    cursor = aws_byte_cursor_from_c_str("\n");
+    ASSERT_TRUE(aws_byte_buf_append_dynamic(&content_buf, &cursor) == AWS_OP_SUCCESS);
+
+    struct aws_string *config_file_contents = aws_string_new_from_array(allocator, content_buf.buffer, content_buf.len);
+    ASSERT_TRUE(config_file_contents != NULL);
+    aws_byte_buf_clean_up(&content_buf);
+
+    s_aws_process_test_init_config_profile(allocator, config_file_contents);
+    aws_string_destroy(config_file_contents);
+
+    aws_unset_environment_value(s_default_profile_env_variable_name);
+
+    struct aws_string *empty_creds = aws_create_process_unique_file_name(allocator);
+    struct aws_string *empty = aws_string_new_from_c_str(allocator, "");
+    aws_create_profile_file(empty_creds, empty);
+    aws_set_environment_value(s_default_credentials_path_env_variable_name, empty_creds);
+    aws_string_destroy(empty_creds);
+    aws_string_destroy(empty);
+
+    struct aws_credentials_provider_profile_options options = {
+        .shutdown_options =
+            {
+                .shutdown_callback = s_on_shutdown_complete,
+                .shutdown_user_data = NULL,
+            },
+    };
+    struct aws_credentials_provider *provider = aws_credentials_provider_new_profile(allocator, &options);
+
+    aws_credentials_provider_get_credentials(provider, s_get_credentials_callback, NULL);
+
+    s_aws_wait_for_credentials_result();
+
+    ASSERT_TRUE(s_tester.has_received_credentials_callback == true);
+    ASSERT_TRUE(s_tester.credentials != NULL);
+    ASSERT_CURSOR_VALUE_STRING_EQUALS(aws_credentials_get_access_key_id(s_tester.credentials), s_profile_access_key_id);
+    ASSERT_CURSOR_VALUE_STRING_EQUALS(
+        aws_credentials_get_secret_access_key(s_tester.credentials), s_profile_secret_access_key);
+
+    aws_credentials_provider_release(provider);
+    s_aws_wait_for_provider_shutdown_callback();
+    s_aws_process_tester_cleanup();
+
+    return 0;
+}
+
+AWS_TEST_CASE(
+    credentials_provider_profile_takes_priority_over_process,
+    s_credentials_provider_profile_takes_priority_over_process);
\ No newline at end of file
